/******************************************************************************/
/* Name:            Juan Gutierrez                                            */
/* Date:            10/10/2008                                                */
/* Project #:                                                                 */
/* Project:                                                                   */
/* Filename:        main.c                                                    */
/* Language:        Embedded C                                                */
/* Keil Version:    uVision 3 V3.50                                           */
/* Target Board:    Olimex LPC-P2148                                          */
/* Osc Freq:        12.000 Mhz                                                */
/******************************************************************************/
#include <LPC21xx.H>
#include "debug_tools.h"

void EXTINTVectoredIRQ (void)__irq;

int main (void)
{
     int i, k;
     //Set the LED pins as outputs
     IODIR0 = 0x00000C00; 
     //Turn OFF the LED pins         
     IOSET0 = 0x00000C00;        

     //Set P0.14 to ENTINT1
     PINSEL0 = 0x20000000;         
     
     /*NOTE:
          VICVectCntl0
          ----------------------------------------------------------------------
          Vector control 0 register. Vector Control Registers 0-15 each control 
          one of the 16 vectored IRQ slots. Slot 0 has the highest priority and 
          slot 15 the lowest.
          
          Refer to page 57, Table 52 of UM10139 PDF for details
     */
     //select a priority slot for a given interrupt
     VICVectCntl0 = 0x0000002F;    
     
     /*Note: 
          Vector Address registers 0-15 (VICVectAddr0-15 - 0xFFFF F100-13C)
          ----------------------------------------------------------------------
          These are a read/write accessible registers. These registers hold the
          addresses of the Interrupt Service routines (ISRs) for the 16 vectored
          IRQ slots.
     */
     //pass the address of the IRQ into the VIC slot
     VICVectAddr0 = (unsigned)EXTINTVectoredIRQ; 

     /*Note:
          Interrupt Enable register (VICIntEnable - 0xFFFF F010)
          ----------------------------------------------------------------------
          When this register is read, 1s indicate interrupt requests or software
          interrupts that are enabled to contribute to FIQ or IRQ. When this 
          register is written, ones enable interrupt requests or software 
          interrupts to contribute to FIQ or IRQ, zeroes have no effect.
     */
     //enable interrupt
     VICIntEnable = 0x00008000;                          
     while(1)
     {
          //Blink LEDs,
          //While P0.14 is brought low an interrupt will occur. In the ISR
          //the buzzer will BEEP. To Exit ISR, bring P0.14 high again.           
          for (i = 0; i < 500000; i++ );		
               //Turn OFF the LED pins         
               IOSET0 = 0x00000C00;
          for (k = 0; k < 500000; k++ );
               //Set the LED pins
               IOCLR0 |= 0x00000C00;                                                                 
     }
}

void EXTINTVectoredIRQ (void)  __irq
{ 
   
     //Beep
     beep(1);                  
     
     /*NOTE:
          EINT1
          -----------------------------------------------------------------
          In level-sensitive mode, this bit is set if the EINT1 function is 
          selected for its pin, and the pin is in its active state. In 
          edge-sensitive mode, this bit is set if the EINT1 function is 
          selected for its pin, and the selected edge occurs on the pin.
     */
     //Clear the peripheral interrupt flag
     EXTINT = 0x00000002;                           

     //Dummy write to signal end of interrupt
     VICVectAddr = 0x00000000;                           
}
